@@ -6,22 +6,26 @@ from attr import attrs
 from xlsxwriter import Workbook
 from xlsxwriter.worksheet import Worksheet
 
-from .formats import FormatHandler
+from formats import FormatHandler
 
 if TYPE_CHECKING:
     from cell_dsl import CommitTypes
+    from ops.classes import WriteRichOp
 
 
 @attrs(auto_attribs=True)
 class WorkbookPair(object):
+    """A pair used to bundle a :class:`FormatHandler` and a :ref:`Workbook <workbook>`"""
     wb: Workbook
     fmt: FormatHandler
 
     def add_worksheet(self, name):
+        """Create a worksheet and bind it into a :class:`WorksheetTriplet`"""
         return WorksheetTriplet(self.wb, self.wb.add_worksheet(name), self.fmt)
 
     @classmethod
     def from_wb(cls, wb):
+        """Bind a :class:`Workbook` into a :class:`WorkbookPair`"""
         return cls(wb, FormatHandler(wb))
 
 
@@ -108,7 +112,7 @@ def col_chain(
 
 def chain_rich(iterable: Iterable['WriteRichOp']) -> 'WriteRichOp':
     """Take an `iterable` of `WriteRich` segments and combine them to produce a single WriteRich operation."""
-    from .ops import WriteRichOp
+    from ops.classes import WriteRichOp
 
     return reduce(WriteRichOp.then, iterable)
 
